/****************************************************************************
*
* FILENAME
*   SQUEUE.H
*
* CLASSNAME
*   SerialQueue
*
* PURPOSE
*   A simple byte oriented queue class with level detection capability.
*
* DESCRIPTION
*   Intended specifically for use by class Serial.
*
* INHERITANCE
*   None
*
* MEMBER FUNCTIONS
*   public:
*   unsigned      SerialQueue::GetCount     ()
*   unsigned char SerialQueue::GetData      ()
*   Boolean       SerialQueue::IsEmpty      ()
*   Boolean       SerialQueue::IsFull       ()
*   Boolean       SerialQueue::IsNearEmpty  ()
*   Boolean       SerialQueue::IsNearFull   ()
*   unsigned char SerialQueue::Peek         (unsigned)
*   Boolean       SerialQueue::Purge        ()
*   void          SerialQueue::PutData      (unsigned char)
*                 SerialQueue::SerialQueue  (SerialQueue &)
*                 SerialQueue::SerialQueue  (unsigned, unsigned, unsigned)
*                 SerialQueue::~SerialQueue ()
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PREPROCESSOR EXCLUSION START
*
*****************************************************************************/

#ifndef __cplusplus
#error Must use C++ for the type SerialQueue.
#endif

#ifndef __LARGE__
#error Must use large memory model for the type SerialQueue.
#endif

#ifndef __SQUEUE.H
#define __SQUEUE.H

/****************************************************************************
*
* GLOBAL HEADER FILES
*
*****************************************************************************/

#include "boolean.h"  // enum Boolean

/***************************************************************************
*
* CLASS DEFINITION
*
*****************************************************************************/

class SerialQueue
{

  private:
  unsigned char *queue;  // Queue data storage area
  unsigned size;         // Size of storage area (bytes)

  unsigned head;         // Index to insert point of queue
  unsigned tail;         // Index to removal point of queue
  unsigned count;        // Bytes currently in the queue

  unsigned full;         // Trigger level for NearFull()
  unsigned empty;        // Trigger level for NearEmpty()

/****************************************************************************
*
* FILENAME
*   SQUEUE.H
*
* FUNCTION NAME
*   SerialQueue::GetCount
*
* PURPOSE
*   Return the number of bytes that are contained in the queue.
*
* DESCRIPTION
*   The private count member is returned.
*
* EXTERNAL REFERENCES
*   class SerialQueue
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

unsigned                 // RETURN: Number of bytes queue contains

SerialQueue::GetCount()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return number of bytes in queue
  return count;
}

/****************************************************************************
*
* FUNCTION NAME
*   SerialQueue::GetData
*
* PURPOSE
*   Remove and return a byte from the queue.
*
* DESCRIPTION
*   If the queue is already empty, then an ASCII NULL is returned.
*
* EXTERNAL REFERENCES
*   class Serial
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

unsigned char           // RETURN: Byte removed from queue

SerialQueue::GetData();

/****************************************************************************
*
* FILENAME
*   SQUEUE.H
*
* FUNCTION NAME
*   SerialQueue::IsEmpty
*
* PURPOSE
*   Returns the Boolean status of the queue.
*
* DESCRIPTION
*   To avoid returning garbage (nulls actually), this function should be
*   before calling GetData().
*
* EXTERNAL REFERENCES
*   class SerialQueue
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

Boolean                 // RETURN: True  = queue is empty
                        //         False = queue is not empty
SerialQueue::IsEmpty()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return True if queue is empty, False otherwise
  return ( count > 0 ) ? False : True;
}

/****************************************************************************
*
* FILENAME
*   SQUEUE.H
*
* FUNCTION NAME
*   SerialQueue::IsFull
*
* PURPOSE
*   Returns the Boolean status of the queue.
*
* DESCRIPTION
*   To ensure data is not lost, this function should be called before any
*   data is added to the queue with PutData().
*
* EXTERNAL REFERENCES
*   class SerialQueue
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

Boolean                // RETURN: True  = Queue is full
                       //         False = Queue is not full

SerialQueue::IsFull()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return True if queue is empty, False otherwise
  return ( count >= size ) ? True : False;
}

/****************************************************************************
*
* FILENAME
*   SQUEUE.H
*
* FUNCTION NAME
*   SerialQueue::IsNearEmpty
*
* PURPOSE
*   Returns the Boolean status of the queue.
*
* DESCRIPTION
*   The trigger level is set by middleSpace in the constructor.
*
* EXTERNAL REFERENCES
*   class SerialQueue
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   June 4, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

Boolean                // RETURN: True  = Queue is nearly empty
                       //         False = Queue is not nearly empty

SerialQueue::IsNearEmpty()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
   // Return True if queue is nearly full, False otherwise
  return (count < empty) ? True : False;
}

/****************************************************************************
*
* FILENAME
*   SQUEUE.H
*
* FUNCTION NAME
*   SerialQueue::IsNearFull
*
* PURPOSE
*   Returns the Boolean status of the queue.
*
* DESCRIPTION
*   The trigger level is set by topSpace in the constructor.
*
* EXTERNAL REFERENCES
*   class SerialQueue
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

Boolean                    // RETURN: True  = Queue is nearly full
                           //         False = Queue is not nearly full

SerialQueue::IsNearFull()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return True if queue nearly full, False otherwise
  return (count > full) ? True : False;
}

/****************************************************************************
*
* FILENAME
*   SQUEUE.CPP
*
* FUNCTION NAME
*   SerialQueue::Peek
*
* PURPOSE
*   Peek at the contents of the queue.
*
* DESCRIPTION
*   The queue is not altered in any way. Attempts to Peek beyond the data
*   in the queue always return ASCII NULL.
*
* EXTERNAL REFERENCES
*   class SerialQueue
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

unsigned char           // RETURN: Byte peeked from queue

SerialQueue::Peek
(
  unsigned offset = 0U  // INPUT: Offset of byte from head of queue
);

/****************************************************************************
*
* FILENAME
*   SQUEUE.H
*
* FUNCTION NAME
*   SerialQueue::Purge
*
* PURPOSE
*   Set the queue to the empty state.
*
* DESCRIPTION
*   All data in the queue is lost.
*
* EXTERNAL REFERENCES
*   class SerialQueue
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   Septermber 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

void                  // RETURN: Nothing is returned

SerialQueue::Purge()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set queue count and indicies to the empty state
  count = head = tail = 0;
}

/****************************************************************************
*
* FILENAME
*   SQUEUE.CPP
*
* FUNCTION NAME
*   SerialQueue::PutData
*
* PURPOSE
*   Put a byte in the queue.
*
* DESCRIPTION
*   If the queue is full already, then the byte is simply ignored (ie. lost).
*
* EXTERNAL REFERENCES
*   class SerialQueue
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

void                  // RETURN: Nothing is returned.

SerialQueue::PutData
(
  unsigned char data  // INPUT: Byte to put into the queue
);

/****************************************************************************
*
* FILENAME
*   SQUEUE.CPP
*
* FUNCTION NAME
*   SerialQueue::SerialQueue
*
* PURPOSE
*   Copy constructor for class SerialQueue.
*
* DESCRIPTION
*   This function is declared, but not defined. This is done so that copying
*   will not take place.
*
* EXTERNAL REFERENCES
*   class SerialQueue
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E.Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

SerialQueue::SerialQueue
(
  SerialQueue &original   // INPUT: Queue to be copy
);

/****************************************************************************
*
* FILENAME
*   SQUEUE.CPP
*
* FUNCTION NAME
*   SerialQueue::SerialQueue
*
* PURPOSE
*   Constructor for class SerialQueue.
*
* DESCRIPTION
*   The size of the queue must be specified; this is the TOTAL number of
*   bytes that will be allocated. The minimum queue size is 256, and the
*   maximum is approximately 65520.
*
*   The trigger levels for NearFull() and NearEmpty() are set by topSpace
*   and middleSpace respectively.
*
* EXTERNAL REFERENCES
*   class SerialQueue
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E.Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

SerialQueue::SerialQueue
(
  unsigned qSize = 256,       // INPUT: Size of queue in bytes
  unsigned topSpace = 50,     // INPUT: Trigger level for NearFull()
  unsigned middleSpace = 150  // INPUT: Trigger level for NearEmpty()
);

/****************************************************************************
*
* FILENAME
*   SQUEUE.CPP
*
* FUNCTION NAME
*   SerialQueue::~SerialQueue
*
* PURPOSE
*   Destructor for class SerialQueue.
*
* DESCRIPTION
*   Memory allocated by the constructor is deallocated.
*
* EXTERNAL REFERENCES
*   class SerialQueue
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: virtual

SerialQueue::~SerialQueue();

/****************************************************************************
*
* CLASS END
*
*****************************************************************************/

};

/****************************************************************************
*
* PREPROCESSOR EXCLUSION END
*
*****************************************************************************/

#endif
